/*---------------------------------------------------------
PROGRAMA: expressao.c

Programa que le uma expressao em notação Polonesa,
cria uma arvore binária que representa a ordem de avaliação dos
operadores e escreve o valor resultante da expressao.

Exemplo (arquivo entrada.txt):
/ - * 5.3 25.7 * 89 11.36 * 3 21

Saida: Valor = -13.886189

Para verificar se há leaks de memoria:
gcc -g expressao.c -o expressao
valgrind --leak-check=full ./expressao < entrada.txt

Para transformar um string em float, utilize a função atof(s):
vFloat= atof( string );

-----------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define OK    1
#define ERRO -1
#define DELIM " "

typedef char TipoItem[10];
typedef struct Nodo *ApNodo;
typedef struct Nodo {
  TipoItem item;
  ApNodo esq, dir;
} Nodo;

/* ----------------------------------------------------- */
/* Impressão da árvore */
void escreveItem( TipoItem v ){
  printf( "%s", v );
}

void escreveNodoInterno( TipoItem v, int h ){
  int i;

  for( i=0; i<h; i++ )
    printf( "   " );
  escreveItem( v );
  printf( "\n" );
}

void escreveNodoExterno( int h ){
  int i;

  for( i=0; i<h; i++ )
    printf( "   " );
  printf( "*\n" );
}

void escreveArv( ApNodo p, int h ){
  if( p == NULL ){
    escreveNodoExterno( h );
    return;
  }
  escreveArv( p->dir, h+1 );
  escreveNodoInterno( p->item, h );
  escreveArv( p->esq, h+1 );
}

/* ----------------------------------------------------- */
/* Função de inicialização de uma árvore vazia */
void inicializa(ApNodo *arv){
  *arv = NULL;
}

/* ----------------------------------------------------- */
/* Criação de um novo nodo com o valor do Item preenchido com v */
ApNodo criaNo( TipoItem v ){
  ApNodo p;

  p = (ApNodo)malloc( sizeof(Nodo) );
  strcpy(p->item, v); p->esq = NULL; p->dir = NULL;
  return p;
}

/* ----------------------------------------------------- */
/* Determina se uma string contem um operador (+ - * / )
   Entrada: string s
   Saida: 1 se s é um operador e 0, caso contrario */
int ehOperador( char *s ){
  if( strcmp(s, "/")== 0 ||
      strcmp(s, "+")== 0 ||
      strcmp(s, "-")== 0 ||
      strcmp(s, "*")== 0 )
    return 1;
  else
    return 0;
}

/* ----------------------------------------------------- */
/* Le um valor da entrada padrão (que pode ser um operador ou uma constante).
   - Se for um operador, insere na árvore com 2 filhos internos 
   - Se form uma constante, insere como uma folha na árvore
   Saida: apontador para o novo nodo 
*/
ApNodo insereItem( ){

}


/* ----------------------------------------------------- 
 Avaliação da expressao 
 Entrada: apontador para um Nodo p
 Saida: valor da expressao da subárvore com raiz no Nodo p
*/
 float avaliaArv( ApNodo p ){

 }

/* ----------------------------------------------------- 
 Função que libera o espaço de memória alocado para a subárvore com raiz em p
*/
void liberaMem(ApNodo p){

}


/* ----------------------------------------------------- */
/* Programa: cria uma árvore binária que representa uma expressao
   aritmetica na notação Polonesa e avalia a expressao.
*/
int main(int argc, char *argv[]){
  ApNodo arv;
     
  inicializa( &arv );
  arv= insereItem( );
  printf( "\nArvore\n" );
  escreveArv( arv, 0 );
  printf("\nValor = %f\n", avaliaArv( arv ));

  liberaMem( arv );
  return 0;
}

